% Examples of using Moran's eigenvector maps (MEM's) in spatial ecology
% 
% by David L. Jones, May-2008
%
% This file is part of the FATHOM Toolbox for Matlab and
% is released under the GNU General Public License, version 2.
% 
% updated June-2012

% File: '.../examples/oribatid_mites.mat''

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%         Oribatid Mite example from Dray's 'spacemakeR for R' manual:         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

% Load data:
load oribatid_mites.mat

% Hellinger transform the species data:
bio.H = f_hellinger(bio.num);

% Dummy code qualitative variables:
env.Qsubstra = f_dummy(env.substra,1);
env.Qschrub  = f_dummy(env.schrub,1);
env.Qmicro   = f_dummy(env.micro,1);

% Perform preliminary RDA on X,Y coordinates to identify linear trends:
result_XY = f_rda(bio.H,[env.x env.y],[0],1000,1);

% Residuals serve as the de-trended data:
bio.dt = result_XY.res;

% Create a Delaunay Triangulation based neighbor graph:
tri = f_delaunay([env.x env.y]);

% Show the binary connectivity matrix ('B'):
f_table(tri.B,'%d','s')

% Plot connectivity graph:
f_plotNeigh(tri,1,'r');
axis([-0.25 2.75 0 10])
title('\bfDelaunay Triangulation');

% Create a weighting matrix ('A') that doesn't change 'B':
A = ones(size(tri.B));

% Create Eigenvector Maps, use 1000 iterations, KEEP neg eigenvalues:
MEM = f_eigenMaps(tri,A,1000,1);

% AIC-based stepwise selection of MEM's:
[best,cond] = f_rdaAIC(bio.dt,MEM.evects,0,1);
% 
% ==================================================
% AIC-based stepwise forward selection (RDA)         
% --------------------------------------------------
% Conditional Tests: (each variable separately)
%     'RSS'       'R2'           'R2adj'          'AIC'        'wts'       'delta'        'ratio'     'var' 
%     [ 18.26]    [ 0.062731]    [   0.048948]    [-89.886]    [     0]    [  0.07393]    [     1]    ' 3'  
%     [18.569]    [ 0.046875]    [   0.032859]    [-88.712]    [1.1743]    [ 0.041098]    [1.7989]    ' 2'  
%     [18.683]    [ 0.041017]    [   0.026914]    [-88.283]    [1.6032]    [ 0.033165]    [2.2291]    ' 4'  
%     [18.712]    [ 0.039548]    [   0.025424]    [-88.175]    [1.7104]    [ 0.031436]    [2.3518]    ' 7'  
%     [18.775]    [ 0.036271]    [   0.022098]    [-87.937]    [1.9488]    [ 0.027902]    [2.6496]    ' 1'  
%     [18.852]    [  0.03235]    [    0.01812]    [-87.653]    [ 2.233]    [ 0.024206]    [3.0542]    ' 5'  
%     [18.854]    [ 0.032237]    [   0.018005]    [-87.645]    [2.2412]    [ 0.024107]    [3.0667]    '16'  
%     [19.482]    [      NaN]    [        NaN]    [-87.471]    [2.4147]    [ 0.022104]    [3.3446]    'none'
%     [18.948]    [ 0.027406]    [   0.013103]    [-87.296]    [2.5898]    [ 0.020251]    [3.6506]    '53'  
%     [19.019]    [ 0.023745]    [  0.0093884]    [-87.033]    [2.8528]    [ 0.017756]    [4.1636]    '10'  
%     [19.064]    [  0.02146]    [    0.00707]    [-86.869]    [3.0164]    [ 0.016361]    [4.5186]    '32'  
%     [19.087]    [ 0.020271]    [  0.0058634]    [-86.784]    [3.1014]    [  0.01568]    [4.7148]    '12'  
%     [  19.1]    [ 0.019615]    [  0.0051975]    [-86.738]    [3.1483]    [ 0.015317]    [4.8266]    '60'  
%     [19.109]    [ 0.019161]    [  0.0047371]    [-86.705]    [3.1807]    [ 0.015071]    [4.9054]    '13'  
%     [19.122]    [ 0.018476]    [  0.0040414]    [-86.656]    [3.2296]    [ 0.014707]    [5.0268]    '58'  
%     [19.144]    [ 0.017369]    [  0.0029187]    [-86.577]    [3.3085]    [ 0.014138]    [ 5.229]    '59'  
%     [19.153]    [ 0.016911]    [  0.0024534]    [-86.545]    [3.3411]    [ 0.013909]    [5.3151]    '28'  
%     [19.162]    [ 0.016441]    [  0.0019769]    [-86.511]    [3.3745]    [ 0.013679]    [5.4047]    '57'  
%     [19.162]    [ 0.016432]    [  0.0019678]    [-86.511]    [3.3752]    [ 0.013674]    [5.4064]    '14'  
%     [19.172]    [ 0.015929]    [  0.0014571]    [-86.475]    [ 3.411]    [ 0.013432]    [5.5041]    '69'  
%     [19.195]    [ 0.014745]    [ 0.00025621]    [-86.391]    [3.4951]    [ 0.012878]    [5.7406]    '23'  
%     [19.196]    [ 0.014676]    [ 0.00018575]    [-86.386]    [3.5001]    [ 0.012847]    [5.7548]    '37'  
%     [19.199]    [ 0.014527]    [ 3.4976e-05]    [-86.375]    [3.5106]    [ 0.012779]    [5.7852]    '15'  
%     [  19.2]    [ 0.014495]    [ 2.2514e-06]    [-86.373]    [3.5129]    [ 0.012764]    [5.7918]    '44'  
%     [19.202]    [ 0.014401]    [-9.2702e-05]    [-86.366]    [3.5195]    [ 0.012722]    [5.8111]    '34'  
%     [19.202]    [ 0.014389]    [-0.00010522]    [-86.365]    [3.5204]    [ 0.012717]    [5.8137]    ' 8'  
%     [19.213]    [ 0.013808]    [-0.00069464]    [-86.324]    [3.5617]    [ 0.012457]    [5.9348]    '33'  
%     [19.221]    [ 0.013383]    [ -0.0011261]    [-86.294]    [3.5918]    [  0.01227]    [ 6.025]    '18'  
%     [19.223]    [ 0.013277]    [ -0.0012336]    [-86.286]    [3.5994]    [ 0.012224]    [6.0477]    '55'  
%     [19.225]    [   0.0132]    [ -0.0013122]    [-86.281]    [3.6049]    [ 0.012191]    [6.0644]    '49'  
%     [ 19.23]    [ 0.012937]    [ -0.0015786]    [-86.262]    [3.6235]    [ 0.012078]    [6.1211]    '17'  
%     [19.249]    [  0.01198]    [ -0.0025495]    [-86.195]    [3.6913]    [ 0.011675]    [6.3322]    '64'  
%     [19.252]    [  0.01179]    [ -0.0027424]    [-86.181]    [3.7048]    [ 0.011597]    [ 6.375]    '25'  
%     [19.272]    [ 0.010784]    [ -0.0037631]    [ -86.11]    [ 3.776]    [ 0.011191]    [6.6061]    '26'  
%     [19.275]    [  0.01065]    [ -0.0038989]    [  -86.1]    [3.7855]    [ 0.011138]    [6.6374]    '62'  
%     [19.278]    [  0.01048]    [ -0.0040715]    [-86.088]    [3.7975]    [ 0.011072]    [6.6775]    '38'  
%     [ 19.28]    [ 0.010385]    [ -0.0041678]    [-86.082]    [3.8042]    [ 0.011034]    [6.6999]    '67'  
%     [19.288]    [ 0.009945]    [ -0.0046146]    [ -86.05]    [3.8353]    [ 0.010864]    [6.8051]    '50'  
%     [19.289]    [0.0099121]    [  -0.004648]    [-86.048]    [3.8377]    [ 0.010851]    [ 6.813]    '31'  
%     [19.289]    [0.0099046]    [ -0.0046556]    [-86.048]    [3.8382]    [ 0.010848]    [6.8148]    ' 6'  
%     [19.291]    [0.0098037]    [  -0.004758]    [ -86.04]    [3.8453]    [  0.01081]    [6.8392]    '61'  
%     [19.293]    [0.0096999]    [ -0.0048633]    [-86.033]    [3.8527]    [  0.01077]    [6.8643]    '41'  
%     [19.294]    [0.0096325]    [ -0.0049317]    [-86.028]    [3.8574]    [ 0.010745]    [6.8807]    '46'  
%     [19.296]    [0.0095776]    [ -0.0049874]    [-86.025]    [3.8613]    [ 0.010724]    [ 6.894]    '40'  
%     [19.297]    [0.0094754]    [ -0.0050912]    [-86.017]    [3.8685]    [ 0.010685]    [ 6.919]    '43'  
%     [19.302]    [0.0092621]    [ -0.0053076]    [-86.002]    [3.8836]    [ 0.010605]    [6.9713]    '19'  
%     [19.303]    [0.0091946]    [ -0.0053761]    [-85.997]    [3.8884]    [  0.01058]    [ 6.988]    '20'  
%     [19.305]    [0.0090977]    [ -0.0054744]    [-85.991]    [3.8952]    [ 0.010543]    [7.0119]    '56'  
%     [19.308]    [0.0089529]    [ -0.0056213]    [ -85.98]    [3.9055]    [  0.01049]    [7.0479]    '27'  
%     [19.318]    [ 0.008408]    [ -0.0061743]    [-85.942]    [3.9439]    [  0.01029]    [7.1848]    '65'  
%     [19.319]    [0.0083891]    [ -0.0061934]    [-85.941]    [3.9453]    [ 0.010283]    [7.1896]    '22'  
%     [19.321]    [0.0082435]    [ -0.0063412]    [ -85.93]    [3.9555]    [  0.01023]    [7.2266]    ' 9'  
%     [19.324]    [ 0.008097]    [ -0.0064898]    [ -85.92]    [3.9659]    [ 0.010177]    [7.2641]    '45'  
%     [19.327]    [0.0079463]    [ -0.0066428]    [-85.909]    [3.9765]    [ 0.010123]    [7.3028]    '63'  
%     [19.328]    [0.0079346]    [ -0.0066546]    [-85.908]    [3.9773]    [ 0.010119]    [7.3058]    '21'  
%     [19.329]    [0.0078366]    [  -0.006754]    [-85.902]    [3.9843]    [ 0.010084]    [7.3311]    '30'  
%     [19.338]    [0.0073712]    [ -0.0072263]    [-85.869]    [4.0171]    [0.0099202]    [7.4524]    '68'  
%     [19.342]    [0.0071782]    [ -0.0074222]    [-85.855]    [4.0307]    [0.0098529]    [7.5033]    '39'  
%     [19.345]    [0.0070243]    [ -0.0075783]    [-85.844]    [4.0415]    [0.0097996]    [7.5441]    '48'  
%     [19.349]    [ 0.006838]    [ -0.0077673]    [-85.831]    [4.0547]    [0.0097355]    [7.5938]    '11'  
%     [19.354]    [0.0065641]    [ -0.0080453]    [-85.812]    [ 4.074]    [ 0.009642]    [7.6675]    '66'  
%     [19.355]    [0.0065228]    [ -0.0080872]    [-85.809]    [4.0769]    [ 0.009628]    [7.6786]    '54'  
%     [19.356]    [0.0064833]    [ -0.0081272]    [-85.806]    [4.0797]    [0.0096146]    [7.6893]    '29'  
%     [19.356]    [0.0064655]    [ -0.0081453]    [-85.805]    [4.0809]    [0.0096086]    [7.6942]    '24'  
%     [19.369]    [0.0057963]    [ -0.0088244]    [-85.758]    [4.1281]    [0.0093848]    [7.8776]    '52'  
%     [19.372]    [  0.00564]    [  -0.008983]    [-85.747]    [4.1391]    [0.0093333]    [7.9211]    '36'  
%     [19.372]    [0.0056314]    [ -0.0089917]    [-85.746]    [4.1397]    [0.0093305]    [7.9235]    '51'  
%     [19.378]    [0.0053302]    [ -0.0092973]    [-85.725]    [4.1609]    [0.0092321]    [8.0079]    '47'  
%     [19.388]    [0.0048506]    [  -0.009784]    [-85.691]    [4.1946]    [0.0090776]    [8.1442]    '35'  
%     [19.427]    [0.0028366]    [  -0.011828]    [ -85.55]    [4.3361]    [0.0084575]    [8.7414]    '42'  
% 
% --------------------------------------------------
% 
% Marginal Tests: (sequential variable addition)
%     'RSS'      'R2'          'R2adj'       'AIC'        'wts'         'deltaN'    'var'     'idx'
%     [18.26]    [0.062731]    [0.048948]    [-89.886]    [ 0.07393]    [2.4147]    ' 3'      [  3]
%     [18.26]    [     NaN]    [     NaN]    [-89.886]    [0.023613]    [1.4069]    'none'       []
% 
% --------------------------------------------------
% 
% RSS   = residual sum-of-squares 
% R2    = fraction of total variance explained 
% R2adj = fraction of adjusted total variance explained 
% AIC   = corrected AIC 
% deltaN = delta associated with NO variable addition 
% wts   = AIC weights 
% var   = variable labels 
% idx   = index to selected variables 
% 
% (Note: RSS, R2, and R2adj in Marginal tests are CUMULATIVE) 

% Show which MEM's were selected as the 'optimal model:
best.idx
% ans = 3

% Note: the example in the 'spacemakeR' manual shows slightly different
% eigenvectors were selected (i.e., 3 2 7 1 16). This is due to slight
% differences in the values of the eigenvectors returned by 'R' vs. Matlab,
% resulting differences in AIC, and differences in precision and rounding
% between the two platforms.
