function  result = f_utm2deg(xx,yy,utmzone)
% - convert vectors of UTM coordinates into Lat/Lon vectors (WGS84)
%
% USAGE: result = f_utm2deg(x,y,utmzone);
%
% Inputs:
%    x, y , utmzone.
%
% result = structure with the following fields:
%  .lon = Longitude vector.  Degrees.  +ddd.ddddd  WGS84
%  .lat = Latitude vector.   Degrees.  +ddd.ddddd  WGS84
%
% SEE ALSO: f_deg2utm

% Description: Function to convert vectors of UTM coordinates into Lat/Lon vectors (WGS84).
% Some code has been extracted from UTMIP.m function by Gabriel Ruiz Martinez.
%
% Inputs:
%    x, y , utmzone.
%
% Outputs:
%    Lat: Latitude vector.   Degrees.  +ddd.ddddd  WGS84
%    Lon: Longitude vector.  Degrees.  +ddd.ddddd  WGS84
%

% Example 1:
% x=[ 458730;  407652;  239026;  230253;  343898;  362850];
% y=[4462881; 5126289; 4163082; 3171843; 4302284; 2772478];
% utmzone=['30 T'; '32 T'; '11 S'; '28 R'; '15 S'; '51 R'];
% [Lat, Lon]=f_utm2deg(x,y,utmzone)
%Lat =
%   40.31543
%   46.28399
%   37.57782
%   28.64565
%   38.85554
%   25.06188
%Lon =
%   -3.48572
%    7.80122
% -119.95536
%  -17.75954
%  -94.79902
%  121.64037
%
% Example 2: Lat/Lon coordinates in Degrees, Minutes and Seconds
%    LatDMS=dms2mat(deg2dms(Lat));
%
% Author: Rafael Palacios. Universidad Pontificia Comillas.  Apr/06
%-------------------------------------------------------------------------

% -----Edits by David L. Jones:-----
% Mar-2011: edited documentation, changed '|' to '||'
% Jun-2011: edited error msg; preallocate Lat/Lon; str2num replaced with
%           str2double; commented out unused variables; replaced x with X
%           and y with Y; resize utmzone if only one row;
% 
%
% This file is part of the FATHOM Toolbox for Matlab.

% -----Check inputs:-----
n1 = length(xx);
n2 = length(yy);

% If only 1 utmzone provided, replicate to match UTM coordinates
if size(utmzone,1)
   utmzone = repmat(utmzone,n1,1);
end
n3 = size(utmzone,1);

if (n1~=n2 || n1~=n3)
   error('x,y and utmzone vectors should have the same number of rows!\n');
end
c = size(utmzone,2);
if (c~=4)
   error('UTMZONE should be a vector of strings like ''30 T''!\n');
end
% -----------------------

result.lon = zeros(n1,1); % preallocate
result.lat = zeros(n1,1); 

% These were pulled out of the below loop (only calculate once):
sa         = 6378137.000000;
sb         = 6356752.314245;
% e        = ( ( ( sa ^ 2 ) - ( sb ^ 2 ) ) ^ 0.5 ) / sa;
e2         = ( ( ( sa ^ 2 ) - ( sb ^ 2 ) ) ^ 0.5 ) / sb;
e2cuadrada = e2 ^ 2;
c          = ( sa ^ 2 ) / sb;
alfa       = ( 3 / 4 ) * e2cuadrada;
beta       = ( 5 / 3 ) * alfa ^ 2;
gama       = ( 35 / 27 ) * alfa ^ 3;
% alpha    = ( sa - sb ) / sa;             %f
% ablandamiento = 1 / alpha;   % 1/f

for i=1:n1
   if (utmzone(i,4)>'X' || utmzone(i,4)<'C')
      error('UTMZONE should be a vector of strings like ''30 T'', not ''30 t''n');
   end
   if (utmzone(i,4)>'M')
      hemis = 'N'; % Northern hemisphere
   else
      hemis = 'S'; % Southern hemisphere
   end
   
   X    = xx(i);
   Y    = yy(i);
   zone = str2double(utmzone(i,1:2));
   % la   = Lat(i);
   % lo   = Lon(i);
      
   X = X - 500000;
   if isequal(hemis,'S'), Y = Y - 10000000; end
      
   S         = ( ( zone * 6 ) - 183 );
   lat       =  Y / ( 6366197.724 * 0.9996 );
   v         = ( c / ( ( 1 + ( e2cuadrada * ( cos(lat) ) ^ 2 ) ) ) ^ 0.5 ) * 0.9996;
   a         = X / v;
   a1        = sin( 2 * lat );
   a2        = a1 * ( cos(lat) ) ^ 2;
   j2        = lat + ( a1 / 2 );
   j4        = ( ( 3 * j2 ) + a2 ) / 4;
   j6        = ( ( 5 * j4 ) + ( a2 * ( cos(lat) ) ^ 2) ) / 3;
   Bm        = 0.9996 * c * ( lat - alfa * j2 + beta * j4 - gama * j6 );
   b         = ( Y - Bm ) / v;
   Epsi      = ( ( e2cuadrada * a^ 2 ) / 2 ) * ( cos(lat) )^ 2;
   Eps       = a * ( 1 - ( Epsi / 3 ) );
   nab       = ( b * ( 1 - Epsi ) ) + lat;
   senoheps  = ( exp(Eps) - exp(-Eps) ) / 2;
   Delt      = atan(senoheps / (cos(nab) ) );
   TaO       = atan(cos(Delt) * tan(nab));
   
   longitude = (Delt *(180 / pi ) ) + S;
   latitude  = ( lat + ( 1 + e2cuadrada* (cos(lat)^ 2) - ( 3 / 2 )...
      * e2cuadrada * sin(lat) * cos(lat) * ( TaO - lat ) )...
      * ( TaO - lat ) ) *  (180 / pi);
   
   result.lon(i) = longitude;
   result.lat(i) = latitude;   
end
